import logging
import os
from rich.logging import RichHandler
from rich.console import Console
from rich.theme import Theme

console = Console(
    theme=Theme(
        {
            "logging.level.info": "cornflower_blue",
            "logging.level.debug": "dark_sea_green4",
            "logging.level.warning": "gold3",
            "logging.level.error": "red1",
        }
    ),
)

FORMAT = "%(message)s"
LOG_LEVEL = os.environ.get("LOG_LEVEL", logging.INFO)

logging.basicConfig(
    format=FORMAT,
    datefmt="[%X]",
    handlers=[RichHandler(markup=True, console=console, rich_tracebacks=True)],
)

logger = logging.getLogger("tono")
logger.setLevel(LOG_LEVEL)



from tono.lib.base import TonoToolFormatter


class ToolFormatter(TonoToolFormatter):
    def format(self, parsed_doc):
        tool = {
            "type": "function",
            "function": {
                "name": parsed_doc.name,
                "description": f"{parsed_doc.short_description} {parsed_doc.long_description}",
            },
        }

        if parsed_doc.params:
            tool["function"]["parameters"] = {
                "type": "object",
                "properties": {
                    param.arg_name: {
                        "type": param.type_name,
                        "description": param.description,
                    }
                    for param in parsed_doc.params
                },
                "required": parsed_doc.required,
                "additionalProperties": False,
            }

            for param in parsed_doc.params:
                if param.enum:
                    tool["function"]["parameters"]["properties"][param.arg_name][
                        "enum"
                    ] = param.enum

        return tool



from abc import ABC, abstractmethod
from typing import Any


class TonoToolFormatter(ABC):
    @abstractmethod
    def format(self, parsed_doc) -> dict:
        pass


class TonoCompletionClient(ABC):
    @abstractmethod
    def tool_formatter(self) -> TonoToolFormatter:
        pass

    @abstractmethod
    def generate_completion(self, messages: list, tools: list, **kwargs) -> tuple:
        pass

    @abstractmethod
    def get_tool_calls(self, response: str) -> list:
        pass

    @abstractmethod
    def get_tool_details(self, tool: Any) -> tuple:
        pass

    @abstractmethod
    def get_response_text(self, response: str) -> str:
        pass

    @abstractmethod
    def format_message(self, message: str, role: str) -> dict:
        pass



import inspect
from typing_extensions import get_type_hints
from docstring_parser import parse
from typing import Callable, Literal, NotRequired, Optional
from dataclasses import dataclass
from tono.lib.base import TonoToolFormatter


@dataclass
class StructuredParam:
    arg_name: str
    type_name: str
    required: bool
    description: Optional[str] = None
    enum: Optional[list] = None


@dataclass
class StructuredDocstring:
    name: str
    short_description: str
    long_description: str
    params: list
    required: list


def analyze_kwargs_from_func(func):
    # Get the function signature and type hints
    sig = inspect.signature(func)
    type_hints = get_type_hints(func)

    # Extract the type of **kwargs
    for param in sig.parameters.values():
        if (
            param.kind == param.VAR_KEYWORD
        ):  # Check for **kwargs (variadic keyword arguments)
            # Get the type hint for kwargs, assuming it's a TypedDict
            typed_dict_type = type_hints.get(param.name)
            if typed_dict_type and hasattr(typed_dict_type, "__annotations__"):
                return analyze_typed_dict(typed_dict_type)

    return []


def is_enum(type_hint):
    if hasattr(type_hint, "__origin__") and type_hint.__origin__ is Literal:
        return True
    return False


# TODO: Add better support for nested Objects and AnyOf
def get_json_type(type_hint):
    if type_hint is str:
        return "string"
    elif type_hint is int:
        return "integer"
    elif type_hint is float:
        return "number"
    elif type_hint is bool:
        return "boolean"
    elif type_hint is list:
        return "array"
    elif type_hint is dict:
        return "object"
    elif type_hint is None:
        return "null"
    else:
        return "object"


def analyze_typed_dict(typed_dict):
    params = []

    # Iterate through annotations and check for NotRequired
    for key, value in typed_dict.__annotations__.items():
        param = {
            "arg_name": key,
            "required": True,
        }
        param["type_name"] = get_json_type(value)

        if is_enum(value):
            param["type_name"] = list(
                set([get_json_type(type(item)) for item in value.__args__])
            )
            param["enum"] = [item for item in value.__args__]

        if hasattr(value, "__origin__") and value.__origin__ is NotRequired:
            param["required"] = False

        params.append(param)

    return params


def parse_docstring(func: Callable, formatter: TonoToolFormatter):
    doc = parse(str(func.__doc__))
    params = analyze_kwargs_from_func(func)

    # update params with descriptions from docstring if available
    for doc_param in doc.params:
        for param in params:
            if doc_param.arg_name == param["arg_name"]:
                param["description"] = doc_param.description

    structured_docstring = StructuredDocstring(
        name=func.__name__,
        short_description=doc.short_description or "",
        long_description=doc.long_description or "",
        params=[StructuredParam(**param) for param in params],
        required=[param["arg_name"] for param in params if param["required"]],
    )

    formatted_doc = formatter.format(structured_docstring)
    return formatted_doc



from pathlib import Path
from datetime import datetime
from typing import TypedDict, Unpack
from tono.lib._logging import logger
from tono.lib._rich import get_input_panel


class TWriteCodeToFile(TypedDict):
    code: str
    file_name: str


def write_code_to_file(**kwargs: Unpack[TWriteCodeToFile]) -> str:
    """Writes the generated code to a file in the current working directory. We are using the w+ mode to write the file.

    :param code: The generated code to be written to a file. Should be plain text and not include code fences.
    :type code: str
    :param file_name: The name of the file to write the code to. Including the file extension.
    :type file_name: str

    :return: A message indicating the status of the operation.
    """
    supported_languages: list[str] = ["python"]
    code = kwargs.get("code", None)
    file_name = kwargs.get(
        "file_name", f"{datetime.now().strftime("%Y%m%d_%H%M%S")}_created_by_tono.py"
    )

    if not file_name.endswith(".py"):
        return f"Invalid language provided. Only the following languages are supported: {supported_languages}. Please rewrite the code in a supported language."

    if not code:
        return "code or file_name not provided to write code to file."

    logger.info(f"Writing code to file {file_name}")
    with open(Path(file_name), "w+") as f:
        f.write(code)
        return f"Code written to {file_name}"


class TWriteToFile(TypedDict):
    text: str
    file_name: str


def write_to_file(**kwargs: Unpack[TWriteToFile]) -> str:
    """Writes the provided text to a file in the current working directory. We are using the w+ mode to write the file.

    :param text: The text to write to the file.
    :param file_name: The name of the file to write the text to. Including the file extension.

    :return: A message indicating the status of the operation.
    :rtype: str
    """
    text = kwargs.get("text", None)
    file_name = kwargs.get("file_name", "created_by_agent.txt")
    logger.debug(f"write_to_file was called with {text=} and {file_name=}")

    if not text:
        return "text or file_name not provided to write to file."

    logger.info(f"Writing text to file {file_name}")
    with open(Path(file_name), "w+") as f:
        f.write(text)
        return f"Text written to {file_name}"


class TGetUserInput(TypedDict):
    question: str


def get_user_input(**kwargs: Unpack[TGetUserInput]) -> str:
    """Prompts the user for input. Use this tool to get input from the user when needed.

    :param str question: The question to ask the user for input.

    :return: The response from the user.
    :rtype: str
    """
    question = kwargs.get("question", None)

    if not question:
        return "No question provided to get user input for."

    response = get_input_panel(question, title="Question?", response_text="Response")
    return response



import os
import anthropic
from tono import Agent
from tono.models.anthropic import CompletionClient
from tono.tools import http_request, write_to_file

anthropic_client = anthropic.Anthropic(
    api_key=os.environ.get("ANTHROPIC_API_KEY"),
)
client = CompletionClient(client=anthropic_client)

# agent creation
agent = Agent(
    name="claude-agent",
    client=client,
    tools=[write_to_file, http_request],
    context=[
        {
            "role": "assistant",
            "content": "You are a helpful assistant that can visit websites by making http requests and write text to files. Use the supplied tools to complete the objective.",
        }
    ],
)

# run the agent
agent.start(
    objective="Check what is trending by visiting a popular news website, and write a summary of the top 10 news articles to a markdown file."
)



import typer
from rich import print

# from tono.cli.commands import init, start
from tono import __version__

app = typer.Typer(name="tono", rich_markup_mode="markdown")
# app.command(short_help="Scaffold new project.")(init.init)
# app.command(short_help="Start the agent.")(start.start)


def version_callback(value: bool):
    if value:
        print(f"Tono CLI Version:  [dark_orange]{__version__}[/dark_orange]")
        raise typer.Exit()


@app.callback()
def common(
    ctx: typer.Context,
    version: bool = typer.Option(None, "--version", callback=version_callback),
):
    pass


if __name__ == "__main__":
    app()



import json
import logging
import uuid
from typing import Callable, Literal
from tono.lib.base import TonoCompletionClient
from tono.lib._rich import get_input_panel
from tono.lib._logging import logger
from tono.lib._docstrings import parse_docstring


class Agent:
    def __init__(
        self,
        client: TonoCompletionClient,
        tools: list[Callable],
        context: list = [],
        name: str = "",
    ):
        self.busy = False
        self.client = client
        self.tools = tools
        self.context = context
        self.tool_definitions = []

        # derive tool definitions from the tools
        for func in tools:
            tool_formatter = self.client.tool_formatter()
            formatted_doc = parse_docstring(func, tool_formatter)
            self.tool_definitions.append(formatted_doc)

        if name == "":
            self.name = str(uuid.uuid4())
        else:
            self.name = name

    def add_to_context(self, message: str, role: Literal["user", "assistant"] = "user"):
        formatted = self.client.format_message(message, role)
        logger.debug(f"Adding message to context: {formatted}")
        if role and message:
            self.context.append(formatted)

    def start(self, objective: str):
        try:
            logger.info(f"Starting agent: [gold3]{self.name}[/gold3]")
            if logger.level > logging.DEBUG:
                logger.info(
                    "Set the log level to [dark_sea_green4]DEBUG[/dark_sea_green4] for more detailed logs."
                )
            self.busy = True

            logger.debug(
                f"Tool Definitions: {json.dumps(self.tool_definitions, indent=2)}"
            )
            self.add_to_context(message=f"Your objective is: {objective}")

            logger.info("Making initial completion request.")
            _, message, tool_calls = self.client.generate_completion(
                messages=self.context,
                tools=self.tool_definitions,
            )

            while self.busy:
                if tool_calls == []:
                    logger.info("No obvious action to be taken.")
                    # prompt the user for an additional prompt
                    user_input = get_input_panel(
                        """Please enter an additional prompt if you would like to give the agent more context. If you have no additional prompt, you can type "exit" to stop the agent.""",
                        title="Additional Prompt",
                        response_text="Additional Prompt",
                    )

                    if user_input.lower() != "exit":
                        self.add_to_context(user_input)
                        _, message, tool_calls = self.client.generate_completion(
                            messages=self.context,
                            tools=self.tool_definitions,
                        )
                        self.add_to_context(message)
                    else:
                        self.busy = False
                        break

                for tool_call in tool_calls:
                    # TODO: Make sure this becomes model agnostic
                    name, kwargs = self.client.get_tool_details(tool_call)

                    # get the tool from the tools list by name
                    selected_tool = next(
                        (tool for tool in self.tools if tool.__name__ == name), None
                    )

                    if selected_tool is None:
                        logger.error(
                            f"Could not find tool: {name}. Please make sure the tool is defined in the tools list."
                        )
                        self.busy = False
                        break

                    logger.info(f"Calling tool: {name} with arguments: {kwargs}")
                    function_call_output = selected_tool(**kwargs)
                    self.add_to_context(
                        f"Called tool: {name} with arguments: {kwargs}. The function call output was: {function_call_output}"
                    )

                logger.debug(json.dumps(self.context, indent=2))
                _, message, tool_calls = self.client.generate_completion(
                    messages=self.context,
                    tools=self.tool_definitions,
                )
                self.add_to_context(message)

        except Exception as e:
            logger.exception(e)



from typing import Optional, Literal, Unpack, Mapping, Any, TypedDict
from bs4 import BeautifulSoup
import httpx


class THTTPRequest(TypedDict):
    url: str
    method: Literal["GET", "POST", "PUT", "DELETE", "PATCH", "OPTIONS"]
    headers: Optional[dict]
    data: Optional[Mapping[str, Any]]


def http_request(**kwargs: Unpack[THTTPRequest]):
    """Send a HTTP request to a server and return the response.
    :param url: The URL of the server.
    :param method: The HTTP method to use. Default is GET.
    :param headers: The headers to send with the request.
    :param data: The data to send with the request.

    :return: The response from the server.

    """
    url = kwargs.get("url", "")
    if not url:
        return "URL is required"
    method = kwargs.get("method", "GET")
    headers = kwargs.get(
        "headers",
        {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3"  # noqa
        },
    )
    data = kwargs.get("data", {})

    try:
        r = httpx.request(method, url, headers=headers, data=data)
        # use beautiful soup to extract the body only
        soup = BeautifulSoup(r.content, "html.parser")

        # delete all the script tags
        for script in soup.find_all("script"):
            script.extract()

        content = str(soup)
        return content
    except Exception as e:
        return e



import pytest
import json
from unittest.mock import Mock, patch
from openai import OpenAI
from tono.models.openai._client import CompletionClient
from tono.models.openai._formatter import ToolFormatter
from tono.lib.base import TonoCompletionClient, TonoToolFormatter


@pytest.fixture
def mock_openai_client():
    return Mock(spec=OpenAI)


@pytest.fixture
def completion_client(mock_openai_client):
    return CompletionClient(client=mock_openai_client, model="gpt-4", temperature=0.3)


def test_subclassed():
    assert issubclass(CompletionClient, TonoCompletionClient)
    assert issubclass(ToolFormatter, TonoToolFormatter)


def test_init_validates_client_type():
    with pytest.raises(TypeError, match="client must be an instance of openai.OpenAI"):
        CompletionClient(client="not_a_client")


def test_tool_formatter_property(completion_client):
    assert isinstance(completion_client.tool_formatter, ToolFormatter)


def test_format_message(completion_client):
    message = "test message"
    formatted = completion_client.format_message(message, role="user")
    assert formatted == {"role": "user", "content": "test message"}
    formatted = completion_client.format_message(message, role="assistant")
    assert formatted == {"role": "assistant", "content": "test message"}

    with pytest.raises(ValueError, match="role must be either 'user' or 'assistant'"):
        completion_client.format_message(message, role="invalid")


def test_get_response_text(completion_client):
    mock_response = json.dumps({"choices": [{"message": {"content": "test response"}}]})
    assert completion_client.get_response_text(mock_response) == "test response"


def test_get_tool_calls_no_tools(completion_client):
    mock_response = json.dumps({"choices": [{"message": {"content": "test"}}]})
    assert completion_client.get_tool_calls(mock_response) == []


def test_get_tool_calls_with_tools(completion_client):
    mock_tool_calls = [{"function": {"name": "test_func", "arguments": "{}"}}]
    mock_response = json.dumps(
        {"choices": [{"message": {"tool_calls": mock_tool_calls}}]}
    )
    assert completion_client.get_tool_calls(mock_response) == mock_tool_calls


def test_get_tool_details(completion_client):
    mock_tool = {"function": {"name": "test_func", "arguments": '{"arg": "value"}'}}
    name, kwargs = completion_client.get_tool_details(mock_tool)
    assert name == "test_func"
    assert kwargs == {"arg": "value"}


@patch("tono.models.openai._client.print_in_panel")
def test_log_completion(mock_print, completion_client):
    mock_response = json.dumps(
        {
            "choices": [
                {
                    "message": {
                        "content": "test response",
                        "tool_calls": [
                            {
                                "function": {
                                    "name": "test_func",
                                    "arguments": '{"arg": "value"}',
                                }
                            }
                        ],
                    }
                }
            ]
        }
    )

    completion_client.log_completion(mock_response)

    mock_print.assert_any_call("test response", title="Agent Message")
    mock_print.assert_any_call(
        "test_func({'arg': 'value'})", title="Tool Call Requested"
    )



from tono.lib.base import TonoToolFormatter


class ToolFormatter(TonoToolFormatter):
    def format(self, parsed_doc):
        tool = {
            "name": parsed_doc.name,
            "description": f"{parsed_doc.short_description} {parsed_doc.long_description}",
        }

        if parsed_doc.params:
            tool["input_schema"] = {
                "type": "object",
                "properties": {
                    param.arg_name: {
                        "type": param.type_name,
                        "description": param.description,
                    }
                    for param in parsed_doc.params
                },
                "required": parsed_doc.required,
            }

            for param in parsed_doc.params:
                if param.enum:
                    tool["input_schema"]["properties"][param.arg_name]["enum"] = (
                        param.enum
                    )

        return tool



import pytest
import json
from unittest.mock import Mock, patch
from anthropic import Anthropic
from tono.models.anthropic._client import CompletionClient
from tono.models.anthropic._formatter import ToolFormatter
from tono.lib.base import TonoCompletionClient, TonoToolFormatter


@pytest.fixture
def mock_anthropic_client():
    return Mock(spec=Anthropic)


@pytest.fixture
def completion_client(mock_anthropic_client):
    return CompletionClient(
        client=mock_anthropic_client, model="claude", temperature=0.3
    )


def test_subclassed():
    assert issubclass(CompletionClient, TonoCompletionClient)
    assert issubclass(ToolFormatter, TonoToolFormatter)


def test_init_validates_client_type():
    with pytest.raises(
        TypeError, match="client must be an instance of anthropic.Anthropic"
    ):
        CompletionClient(client="not_a_client")


def test_tool_formatter_property(completion_client):
    assert isinstance(completion_client.tool_formatter, ToolFormatter)


def test_format_message(completion_client):
    message = "test message"
    formatted = completion_client.format_message(message, role="user")
    assert formatted == {"role": "user", "content": "test message"}
    formatted = completion_client.format_message(message, role="assistant")
    assert formatted == {"role": "assistant", "content": "test message"}

    with pytest.raises(ValueError, match="role must be either 'user' or 'assistant'"):
        completion_client.format_message(message, role="invalid")


def test_get_response_text(completion_client):
    mock_response = json.dumps({"content": [{"type": "text", "text": "test response"}]})
    assert completion_client.get_response_text(mock_response) == "test response"


def test_get_tool_calls_no_tools(completion_client):
    mock_response = json.dumps({"content": [{"type": "text", "text": "test"}]})
    assert completion_client.get_tool_calls(mock_response) == []


def test_get_tool_calls_with_tools(completion_client):
    mock_response = json.dumps(
        {
            "role": "assistant",
            "content": [
                {
                    "type": "text",
                    "text": "<thinking>To answer this question, I will: 1. Use the get_weather tool to get the current weather in San Francisco. 2. Use the get_time tool to get the current time in the America/Los_Angeles timezone, which covers San Francisco, CA.</thinking>",
                },
                {
                    "type": "tool_use",
                    "id": "toolu_01A09q90qw90lq917835lq9",
                    "name": "get_weather",
                    "input": {"location": "San Francisco, CA"},
                },
            ],
        }
    )
    assert completion_client.get_tool_calls(mock_response) == [
        {
            "type": "tool_use",
            "id": "toolu_01A09q90qw90lq917835lq9",
            "name": "get_weather",
            "input": {"location": "San Francisco, CA"},
        }
    ]


def test_get_tool_details(completion_client):
    mock_tool = {"name": "test_func", "input": {"arg": "value"}}
    name, kwargs = completion_client.get_tool_details(mock_tool)
    assert name == "test_func"
    assert kwargs == {"arg": "value"}


@patch("tono.models.anthropic._client.print_in_panel")
def test_log_completion(mock_print, completion_client):
    mock_response = json.dumps(
        {
            "role": "assistant",
            "content": [
                {
                    "type": "text",
                    "text": "test response",
                },
                {
                    "type": "tool_use",
                    "id": "toolu_01A09q90qw90lq917835lq9",
                    "name": "get_weather",
                    "input": {"location": "San Francisco, CA"},
                },
            ],
        }
    )

    completion_client.log_completion(mock_response)

    mock_print.assert_any_call("test response", title="Agent Message")
    mock_print.assert_any_call(
        "get_weather({'location': 'San Francisco, CA'})", title="Tool Call Requested"
    )



import sys
import json
from typing import Any, Literal
from tono.lib._rich import print_in_panel
from tono.lib._logging import logger
from tono.lib.base import TonoCompletionClient
from tono.models.openai._formatter import ToolFormatter
from rich import print as rich_print

try:
    import openai
except ImportError:
    rich_print(
        r"""Please run 'pip install "tono\[openai]"' or 'pip install "tono\[all]"' to use the OpenAI models."""
    )
    raise sys.exit(1)


class CompletionClient(TonoCompletionClient):
    def __init__(
        self,
        client: openai.OpenAI,
        model: str = "gpt-4o",
        temperature: float = 0.3,
        **kwargs,
    ):
        if not isinstance(client, openai.OpenAI):
            raise TypeError("client must be an instance of openai.OpenAI")

        self.client = client
        self.model = model
        self.temperature = temperature
        self.kwargs = kwargs

    @property
    def tool_formatter(self):
        return ToolFormatter()

    def generate_completion(
        self,
        messages: list,
        tools: list = [],
        **kwargs,
    ):
        # Pass the merged kwargs to the OpenAI client
        merged_kwargs = {**kwargs, **self.kwargs}
        response = self.client.chat.completions.create(
            model=self.model,
            messages=messages,
            tools=tools,
            temperature=self.temperature,
            **merged_kwargs,
        )
        res_json = response.to_json()
        message = self.get_response_text(res_json)
        tool_calls = self.get_tool_calls(res_json)

        self.log_completion(res_json)

        return response, message, tool_calls

    def get_tool_calls(self, response: str) -> list:
        try:
            tool_calls = json.loads(response)["choices"][0]["message"].get(
                "tool_calls", None
            )
            if tool_calls is None:
                return []
            return tool_calls
        except Exception as e:
            logger.debug(f"Error getting tool calls: {e}")
            return []

    def get_tool_details(self, tool: Any) -> tuple:
        name = tool["function"]["name"]
        kwargs = json.loads(tool["function"]["arguments"])
        return name, kwargs

    def get_response_text(self, response: str) -> str:
        return json.loads(response)["choices"][0]["message"]["content"]

    def format_message(self, message: str, role=Literal["user", "assistant"]) -> dict:
        if role not in ["user", "assistant"]:
            raise ValueError("role must be either 'user' or 'assistant'")

        return {"role": role, "content": str(message)}

    def log_completion(self, response: str):
        content = self.get_response_text(response)
        tool_calls = self.get_tool_calls(response)

        # log info in a panel
        if content:
            print_in_panel(str(content), title="Agent Message")

        if tool_calls:
            # loop through tool calls and format them as python function calls
            for tool_call in tool_calls:
                name = tool_call["function"]["name"]
                kwargs = json.loads(tool_call["function"]["arguments"])
                # truncate any long arguments
                for key, value in kwargs.items():
                    if len(str(value)) > 50:
                        kwargs[key] = f"{str(value)[:50]}..."

                print_in_panel(f"{name}({kwargs})", title="Tool Call Requested")



import tono
import pytest

import tono.models
import tono.models.anthropic
import tono.models.anthropic._formatter
import tono.models.openai
import tono.models.openai._formatter


class MockParam:
    def __init__(self, arg_name, type_name, description, enum=None):
        self.arg_name = arg_name
        self.type_name = type_name
        self.description = description
        self.enum = enum


class MockParsedDoc:
    def __init__(
        self, name, short_description, long_description, params=None, required=None
    ):
        self.name = name
        self.short_description = short_description
        self.long_description = long_description
        self.params = params or []
        self.required = required or []


@pytest.mark.parametrize(
    "formatter,expected",
    [
        (
            tono.models.openai._formatter.ToolFormatter(),
            {
                "type": "function",
                "function": {
                    "name": "test_tool",
                    "description": "A test tool This tool is used for testing.",
                },
            },
        ),
        (
            tono.models.anthropic._formatter.ToolFormatter(),
            {
                "name": "test_tool",
                "description": "A test tool This tool is used for testing.",
            },
        ),
    ],
)
def test_tool_formatter_no_params(formatter, expected):
    parsed_doc = MockParsedDoc(
        name="test_tool",
        short_description="A test tool",
        long_description="This tool is used for testing.",
    )
    tool = formatter.format(parsed_doc)
    assert tool == expected


@pytest.mark.parametrize(
    "formatter,expected",
    [
        (
            tono.models.openai._formatter.ToolFormatter(),
            {
                "type": "function",
                "function": {
                    "name": "test_tool",
                    "description": "A test tool This tool is used for testing.",
                    "parameters": {
                        "type": "object",
                        "properties": {
                            "param1": {
                                "type": "string",
                                "description": "The first parameter",
                            },
                            "param2": {
                                "type": "integer",
                                "description": "The second parameter",
                            },
                        },
                        "required": ["param1", "param2"],
                        "additionalProperties": False,
                    },
                },
            },
        ),
        (
            tono.models.anthropic._formatter.ToolFormatter(),
            {
                "name": "test_tool",
                "description": "A test tool This tool is used for testing.",
                "input_schema": {
                    "type": "object",
                    "properties": {
                        "param1": {
                            "type": "string",
                            "description": "The first parameter",
                        },
                        "param2": {
                            "type": "integer",
                            "description": "The second parameter",
                        },
                    },
                    "required": ["param1", "param2"],
                },
            },
        ),
    ],
)
def test_tool_formatter_with_params(formatter, expected):
    params = [
        MockParam(
            arg_name="param1", type_name="string", description="The first parameter"
        ),
        MockParam(
            arg_name="param2", type_name="integer", description="The second parameter"
        ),
    ]
    parsed_doc = MockParsedDoc(
        name="test_tool",
        short_description="A test tool",
        long_description="This tool is used for testing.",
        params=params,
        required=["param1", "param2"],
    )
    tool = formatter.format(parsed_doc)
    assert tool == expected


@pytest.mark.parametrize(
    "formatter,expected",
    [
        (
            tono.models.openai._formatter.ToolFormatter(),
            {
                "type": "function",
                "function": {
                    "name": "test_tool",
                    "description": "A test tool This tool is used for testing.",
                    "parameters": {
                        "type": "object",
                        "properties": {
                            "param1": {
                                "type": "string",
                                "description": "The first parameter",
                                "enum": ["option1", "option2"],
                            }
                        },
                        "required": ["param1"],
                        "additionalProperties": False,
                    },
                },
            },
        ),
        (
            tono.models.anthropic._formatter.ToolFormatter(),
            {
                "name": "test_tool",
                "description": "A test tool This tool is used for testing.",
                "input_schema": {
                    "type": "object",
                    "properties": {
                        "param1": {
                            "type": "string",
                            "description": "The first parameter",
                            "enum": ["option1", "option2"],
                        }
                    },
                    "required": ["param1"],
                },
            },
        ),
    ],
)
def test_tool_formatter_with_enum(formatter, expected):
    params = [
        MockParam(
            arg_name="param1",
            type_name="string",
            description="The first parameter",
            enum=["option1", "option2"],
        )
    ]
    parsed_doc = MockParsedDoc(
        name="test_tool",
        short_description="A test tool",
        long_description="This tool is used for testing.",
        params=params,
        required=["param1"],
    )
    tool = formatter.format(parsed_doc)
    assert tool == expected



import typer
from tono.lib._logging import console
from rich.panel import Panel


def get_input_panel(
    text: str, title: str = "Question?", response_text: str = "Response"
):
    console.print("\n")
    console.print(
        Panel(
            str(text),
            title=title,
            padding=(1, 1),
            highlight=True,
            border_style="color(49)",
        )
    )
    console.print("\n")
    return typer.prompt(response_text)


def print_in_panel(text: str, title: str = "Output"):
    console.print("\n")
    console.print(
        Panel(
            str(text),
            title=title,
            padding=(1, 1),
            highlight=True,
        )
    )
    console.print("\n")
